package com.akamai.hdcore.samples.hdn1
{
	import com.akamai.hd.HDEvent;
	import com.akamai.hd.HDNetStream;
	import com.akamai.hdcore.samples.utility.HDN1MediaPlayerViewPresenter;
	import com.akamai.hdcore.samples.utility.MediaPlayerView;
	
	import flash.events.TimerEvent;
	import flash.net.NetConnection;
	import flash.utils.Timer;
	
	import org.openvideoplayer.components.ui.controlbar.view.MBRSwitchingToggleButton;
	import org.openvideoplayer.components.ui.shared.ControlType;
	import org.openvideoplayer.components.ui.shared.event.ControlEvent;
	import org.openvideoplayer.components.ui.shared.view.ScrollDownButton;
	import org.openvideoplayer.components.ui.shared.view.ScrollUpButton;
	
	/**
	 * This is a basic presenter class that allows you to override and enhance functionality
	 * within the HDN1MediaPlayerViewPresenter.as class located in the HDCoreSamplesUtility 
	 * project. 
	 * 
	 */
	public class ManualSwitchingSamplePresenter extends HDN1MediaPlayerViewPresenter
	{				
		private static const CAPTION_TIMER_INTERVAL:Number = 4000;
		private static const STREAM_TRANSITION_AT_HIGHEST:String = "Already playing the highest quality stream.";
		private static const STREAM_TRANSITION_AT_LOWEST:String = "Already playing the lowest quality stream.";
		
		private var scrollDownButton:ScrollDownButton;
		private var scrollUpButton:ScrollUpButton;
		private var mbrSwitchingToggleButton:MBRSwitchingToggleButton;
		private var lastSwitch:int = 0;
		private var manualSwitch:Boolean;
		private var captionTimer:Timer;
		private var switchingToIndex:uint;
		

		
		public function ManualSwitchingSamplePresenter(view:MediaPlayerView, netStream:HDNetStream, netConnection:NetConnection)
		{			
			super(view, netStream, netConnection, true/*set to false to hide the buffer metrics*/);
			
			captionTimer = new Timer(CAPTION_TIMER_INTERVAL);
			captionTimer.addEventListener(TimerEvent.TIMER, onCaptionTimer);

			netStream.loop = false;
		}
		
		override public function destroy():void 
		{
			captionTimer.stop();
			captionTimer.removeEventListener(TimerEvent.TIMER, onCaptionTimer);
			captionTimer = null;
			
			super.destroy();
		}		
			
		override protected function onPauseClick(event:ControlEvent = null):void
		{
			super.onPauseClick(event);
			disableSwitchingControls();
		}
		
		override protected function onPlayClick(event:ControlEvent=null):void
		{
			super.onPlayClick(event);
			mbrSwitchingToggleButton.enabled = true;
		}
		
		override protected function setControlReferences():void
		{
			super.setControlReferences();
			mbrSwitchingToggleButton = controlBar.getControl(ControlType.MBR_SWITCH_TOGGLE_BUTTON) as MBRSwitchingToggleButton;
			scrollDownButton = controlBar.getControl(ControlType.SCROLL_DOWN_BUTTON) as ScrollDownButton;
			scrollUpButton = controlBar.getControl(ControlType.SCROLL_UP_BUTTON) as ScrollUpButton;
			disableSwitchingControls();
		}
		
		override protected function addControlBarListeners():void
		{
			super.addControlBarListeners();
			controlBar.addEventListener(ControlEvent.AUTO_MBR, onToggleAutoSwitching);
			controlBar.addEventListener(ControlEvent.MANUAL_MBR, onToggleAutoSwitching);
			controlBar.addEventListener(ControlEvent.SCROLL_UP, onSwitchUp);
			controlBar.addEventListener(ControlEvent.SCROLL_DOWN, onSwitchDown);
		}
		
		override protected function removeControlBarListeners():void
		{
			super.removeControlBarListeners();
			controlBar.removeEventListener(ControlEvent.AUTO_MBR, onToggleAutoSwitching);
			controlBar.removeEventListener(ControlEvent.MANUAL_MBR, onToggleAutoSwitching);
			controlBar.removeEventListener(ControlEvent.SCROLL_UP, onSwitchUp);
			controlBar.removeEventListener(ControlEvent.SCROLL_DOWN, onSwitchDown);
		}
				
		private function onToggleAutoSwitching(event:ControlEvent):void
		{
			if (netStream)
			{
				manualSwitch = !manualSwitch;
				toggleMBRSwitchButtonsEnabled(manualSwitch);
				HDNetStream(netStream).autoSwitch = !manualSwitch
			}
		}
		
		private function onSwitchUp(event:ControlEvent):void
		{
			
			lastSwitch = 1;
			switchingToIndex = HDNetStream(netStream).currentIndex + 1;
			if (switchingToIndex < HDNetStream(netStream).MBRObject.streamCount)
			{
				HDNetStream(netStream).setCurrentIndex(switchingToIndex);
			}
			else
			{
				handleInvalidIndexError();				
			}
		}
		
		private function onSwitchDown(event:ControlEvent):void
		{	
			lastSwitch = -1;
			switchingToIndex = HDNetStream(netStream).currentIndex > 0 ?  HDNetStream(netStream).currentIndex - 1 : 0;
			if (HDNetStream(netStream).currentIndex > 0)
			{
				HDNetStream(netStream).setCurrentIndex(switchingToIndex);
			}
			else
			{
				handleInvalidIndexError();
			}
		}
		
		private function onSwitchStart(event:HDEvent):void
		{
			if (captionTimer.running)
			{
				captionTimer.reset();
			}
			
			showTransitionCaption(getTransistionCaption(event));
			
			toggleMBRSwitchButtonsEnabled(false);
			view.console.log(event.toString());	
		}
		
		private function onSwitchComplete(event:HDEvent):void
		{
			if (manualSwitch)
			{
				toggleMBRSwitchButtonsEnabled(true);
			}
			showTransitionCaption(getTransistionCaption(event));
			captionTimer.start();
			view.console.log(event.toString());
		}
		
		private function getTransistionCaption(event:HDEvent):String
		{
			var transitionCaption:String;
			
			switch (event.type)
			{
				case HDEvent.SWITCH_START:
					transitionCaption = "Switch started to index " + event.data.index + ", " + event.data.reason;
					break;
				case HDEvent.SWITCH_COMPLETE:
					transitionCaption = "Switch complete to index " + event.data + ", bitrate: " + HDNetStream(netStream).currentIndexBitrate;
					break;
			}
			
			return transitionCaption;
		}
		
		private function handleInvalidIndexError():void 
		{
			var msg:String;
			if (lastSwitch > 0) 
			{
				msg = STREAM_TRANSITION_AT_HIGHEST;
			}
			else if (lastSwitch < 0) 
			{
				msg = STREAM_TRANSITION_AT_LOWEST;
			}
			
			showTransitionCaption(msg);
			captionTimer.start();
			
			scrollDownButton.enabled = (lastSwitch > 0);
			scrollUpButton.enabled = (lastSwitch < 1);
		}
		
		private function showTransitionCaption(value:String):void
		{
			MediaPlayerView(view).captioningView.visible = true;
			MediaPlayerView(view).captioningView.label.text = value
		}
	
		private function onCaptionTimer(event:TimerEvent):void
		{
			captionTimer.reset();
			MediaPlayerView(view).captioningView.visible = false;
		}
		
		override protected function setupNetStreamListeners(add:Boolean=true):void
		{
			super.setupNetStreamListeners(add);
			if (add)
			{
				netStream.addEventListener(HDEvent.SWITCH_START, onSwitchStart);
				netStream.addEventListener(HDEvent.SWITCH_COMPLETE, onSwitchComplete);
			}
			else
			{
				netStream.removeEventListener(HDEvent.SWITCH_START, onSwitchStart);
				netStream.removeEventListener(HDEvent.SWITCH_COMPLETE, onSwitchComplete);
			}
		}
		
		override protected function onComplete(event:HDEvent):void
		{
			super.onComplete(event);
			disableSwitchingControls();
		}
		
		private function toggleMBRSwitchButtonsEnabled(value:Boolean):void
		{
			scrollDownButton.enabled = value;
			scrollUpButton.enabled = value;
		}		
		
		private function disableSwitchingControls():void
		{
			toggleMBRSwitchButtonsEnabled(false);
			mbrSwitchingToggleButton.enabled = false;
		}
	}
}
